"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.statementParser = exports.parseCodeStatements = void 0;
var _traverse2 = _interopRequireDefault(require("@babel/traverse"));
var t = _interopRequireWildcard(require("@babel/types"));
var _parser = require("@babel/parser");
var _core = require("@babel/core");
var _transform = require("./transform");
var _defaultBabelConfig = require("../config/default-babel-config");
var _ast = require("./ast");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { "default": e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n["default"] = e, t && t.set(e, n), n; }
function extractVariableNames(pattern) {
  var names = [];
  var _traverse = function traverse(node) {
    if (!node) return;
    switch (node.type) {
      case 'Identifier':
        names.push(node.name);
        break;
      case 'ArrayPattern':
        node.elements.forEach(_traverse);
        break;
      case 'ObjectPattern':
        node.properties.forEach(function (prop) {
          if (prop.type === 'ObjectProperty') {
            // 处理 { a: b } 形式
            if (prop.value.type === 'Identifier') {
              names.push(prop.value.name);
            } else {
              _traverse(prop.value);
            }
          }
          // 处理剩余运算符 ...rest
          else if (prop.type === 'RestElement') {
            _traverse(prop.argument);
          }
        });
        break;
      case 'AssignmentPattern':
        _traverse(node.left);
        break;
      case 'RestElement':
        _traverse(node.argument);
        break;
    }
  };
  _traverse(pattern);
  return names;
}

/**
 * 构造一个本地名→{ package, importName } 的映射
 */
/** 构建本地名→{ package, importName } 映射 */
function buildImportMap(ast) {
  var importMap = {};
  (0, _traverse2["default"])(ast, {
    ImportDeclaration: function ImportDeclaration(path) {
      var pkg = path.node.source.value;
      path.node.specifiers.forEach(function (spec) {
        if (t.isImportSpecifier(spec)) {
          importMap[spec.local.name] = {
            "package": pkg,
            importName: spec.imported.name
          };
        } else if (t.isImportDefaultSpecifier(spec)) {
          importMap[spec.local.name] = {
            "package": pkg,
            importName: spec.local.name,
            "default": true
          };
        } else if (t.isImportNamespaceSpecifier(spec)) {
          importMap[spec.local.name] = {
            "package": pkg,
            importName: spec.local.name,
            namespace: true
          };
        }
      });
    }
  });
  return importMap;
}

/** 在给定的语句路径上收集“真正的”import依赖 */
function collectDeps(stmtPath, importMap) {
  var deps = {};
  stmtPath.traverse({
    Identifier: function Identifier(idPath) {
      var name = idPath.node.name;
      // 只有当这个 name 在 importMap 中，
      // 并且当前 scope 里确实把它当做 “module” 绑定时，
      // 才算是对 import 的引用
      var binding = idPath.scope.getBinding(name);
      if (importMap[name] && binding && binding.kind === 'module') {
        var key = importMap[name]["package"] + "::" + importMap[name].importName;
        deps[key] = importMap[name];
      }
    }
  });

  // 返回去重后的数组
  return Object.values(deps);
}

// 将代码中特定的部分删除 "\n\n// 不支持 return"
function removeUnsupportedComment(code) {
  return code.replace("\n\n// 不支持 return", '');
}
/**
 * 解析代码并提取结构化语句信息
 * @param code 原始代码字符串
 * @param ast 经过转换的AST语法树
 * @returns 包含语句集合和错误信息的对象
 */
var statementParser = exports.statementParser = function statementParser(code, ast) {
  var statements = [];
  var errorsByStatements = {};
  // 第一步：拿到 importMap
  var importMap = buildImportMap(ast);
  (0, _traverse2["default"])(ast, {
    FunctionDeclaration: function FunctionDeclaration(path) {
      var _path$node$id;
      if (((_path$node$id = path.node.id) === null || _path$node$id === void 0 ? void 0 : _path$node$id.name) !== 'Page') return;
      path.get('body.body').forEach(function (stmtPath) {
        var stmt = stmtPath.node;
        var snippet = code.slice(stmt.start, stmt.end);
        // 针对这一条 stmtPath 收集真正的依赖
        var dependencies = collectDeps(stmtPath, importMap);

        // 编译检测
        var tmpAst = (0, _parser.parse)('');
        tmpAst.program.body.push(stmt);
        var transformed = (0, _core.transformFromAstSync)(tmpAst);
        var _transformJS = (0, _transform.transformJS)(transformed.code, _defaultBabelConfig.defaultBabelConfig),
          hasError = _transformJS.hasError,
          errorInfo = _transformJS.errorInfo,
          _transformJS$code = _transformJS.code,
          compiledCode = _transformJS$code === void 0 ? '' : _transformJS$code;
        // 变量声明
        if (t.isVariableDeclaration(stmt)) {
          var decl = stmt.declarations[0];
          var names = extractVariableNames(decl.id);
          statements.push({
            type: 'JsVarDeclaration',
            value: removeUnsupportedComment(compiledCode),
            names: names,
            dependencies: dependencies,
            source: snippet
          });
          if (hasError && errorInfo) {
            errorsByStatements[snippet] = errorInfo;
          }
          return;
        }

        // 表达式语句
        if (t.isExpressionStatement(stmt)) {
          statements.push({
            type: 'JSExpression',
            value: removeUnsupportedComment(compiledCode),
            dependencies: dependencies,
            source: snippet
          });
          if (hasError && errorInfo) {
            errorsByStatements[snippet] = errorInfo;
          }
          return;
        }

        // 函数声明
        if (t.isFunctionDeclaration(stmt)) {
          var name = stmt.id.name;
          statements.push({
            type: 'JSFunction',
            name: name,
            value: removeUnsupportedComment(compiledCode),
            dependencies: dependencies,
            source: snippet
          });
          if (hasError && errorInfo) {
            errorsByStatements[snippet] = errorInfo;
          }
        }
      });
    }
  });
  return {
    statements: statements,
    errorsByStatements: errorsByStatements
  };
};

/**
 * 将原始代码转换为结构化语句信息
 * @param code 需要解析的代码字符串
 * @returns 包含语句集合和错误信息的对象
 */
var parseCodeStatements = exports.parseCodeStatements = function parseCodeStatements(code, imports) {
  var finalCode = imports + "; function Page() { " + code + " }";
  var ast = (0, _ast.transformAst)(finalCode);
  return statementParser(finalCode, ast);
};